
#ifndef KPROPERTYWIDGETS_EXPORT_H
#define KPROPERTYWIDGETS_EXPORT_H

#ifdef KPROPERTYWIDGETS_STATIC_DEFINE
#  define KPROPERTYWIDGETS_EXPORT
#  define KPROPERTYWIDGETS_NO_EXPORT
#else
#  ifndef KPROPERTYWIDGETS_EXPORT
#    ifdef KPropertyWidgets_EXPORTS
        /* We are building this library */
#      define KPROPERTYWIDGETS_EXPORT __declspec(dllexport)
#    else
        /* We are using this library */
#      define KPROPERTYWIDGETS_EXPORT __declspec(dllimport)
#    endif
#  endif

#  ifndef KPROPERTYWIDGETS_NO_EXPORT
#    define KPROPERTYWIDGETS_NO_EXPORT 
#  endif
#endif

#ifndef KPROPERTYWIDGETS_DEPRECATED
#  define KPROPERTYWIDGETS_DEPRECATED __attribute__ ((__deprecated__))
#endif

#ifndef KPROPERTYWIDGETS_DEPRECATED_EXPORT
#  define KPROPERTYWIDGETS_DEPRECATED_EXPORT KPROPERTYWIDGETS_EXPORT KPROPERTYWIDGETS_DEPRECATED
#endif

#ifndef KPROPERTYWIDGETS_DEPRECATED_NO_EXPORT
#  define KPROPERTYWIDGETS_DEPRECATED_NO_EXPORT KPROPERTYWIDGETS_NO_EXPORT KPROPERTYWIDGETS_DEPRECATED
#endif

#if 0 /* DEFINE_NO_DEPRECATED */
#  ifndef KPROPERTYWIDGETS_NO_DEPRECATED
#    define KPROPERTYWIDGETS_NO_DEPRECATED
#  endif
#endif

#endif
